#!/bin/bash

#
#------------------------------------------------------------------------------
# setup

mode=
fast=
verbose=0

while (($#))
do
    if [[ ${1:0:1} == - ]]
    then
	[[ $1 = -f || $1 = --fast    ]] && fast=--fast
	[[ $1 = -v || $1 = --verbose ]] && verbose=1
    else
	mode="$1"
    fi
    shift
done

LANG_LIST="E F G I J K M Q R S U"
FORMAT="    - %-8s %-12s %s\n"
PDIR=files/_Wiimm/bmg-patch
UIDIR=files/Scene/UI
rm -rf "$UIDIR/".*.d/

ALL="$PDIR/all.txt"
[[ -f $ALL ]] || echo "#BMG-TXT" >"$ALL"

BASE="$PDIR/base.txt"
[[ -f $BASE ]] || echo "#BMG-TXT" >"$BASE"

MARK="$PDIR/mark.txt"
[[ -f $MARK ]] || echo "#BMG-TXT" >"$MARK"

ORIG=

PATCH="wbmgt patch --quiet --ignore"

((verbose)) && printf " * patch messages, mode=%s %s\n" "$mode" "$fast"

#
#------------------------------------------------------------------------------
# check mode -> set default language & force

DEF_LANG="$ALL"
for lang in $LANG_LIST
do
    if [[ -s "$PDIR/lang-$lang.txt" ]]
    then
	DEF_LANG="$PDIR/lang-$lang.txt"
	break
    fi
done

FORCE=0
[[ -s "$PDIR/lang-E.txt" ]] && DEF_LANG="$PDIR/lang-E.txt"

if [[ ${mode:0:2} = en && -s "$PDIR/lang-E.txt" ]]
then
    DEF_LANG="$PDIR/lang-E.txt"
    ORIG="$PDIR/orig-E.txt"
    [[ ${mode:2:1} = + ]] && FORCE=1
fi

if [[ ${mode:0:2} = de && -s "$PDIR/lang-G.txt" ]]
then
    DEF_LANG="$PDIR/lang-G.txt"
    ORIG="$PDIR/orig-G.txt"
    [[ ${mode:2:1} = + ]] && FORCE=1
fi

((verbose)) && printf "$FORMAT" mode "force=$FORCE" "$ORIG" "$DEF_LANG"


#--- extract message files

((verbose)) && printf "$FORMAT" extract szs "$UIDIR"
wszst extract --quiet "$UIDIR/"*_?.szs --dest %P/.%N.d


#--- force a language

if ((FORCE)) && [[ -s $ORIG ]]
then
    ((verbose)) && printf "$FORMAT" patch "base=*" \
			"${ORIG##*/}"
    $PATCH "$UIDIR/".*.d/message/*.bmg \
		--patch "replace,$ORIG"
fi


#--- base messages

if [[ ! -s $DEF_LANG ]]
then
    ((verbose)) && printf "$FORMAT" patch "base=*" \
			"${MARK##*/} ${BASE##*/} ${ALL##*/}"
    $PATCH "$UIDIR/".*.d/message/*.bmg \
		--patch "print,$MARK" \
		--patch "replace,$BASE" \
		--patch "replace,$ALL"
else
    ((verbose)) && printf "$FORMAT" patch "base=*" \
			"${MARK##*/} ${BASE##*/} ${DEF_LANG##*/} ${ALL##*/}"
    $PATCH "$UIDIR/".*.d/message/*.bmg \
		--patch "print,$MARK" \
		--patch "replace,$BASE" \
		--patch "replace,$DEF_LANG" \
		--patch "replace,$ALL"

    if ((!FORCE))
    then
	for lang in $LANG_LIST
	do
	    src="$PDIR/lang-$lang.txt"
	    if [[ -s $src && $src != $DEF_LANG ]]
	    then
		((verbose)) && printf "$FORMAT" patch "base=$lang" \
			"${src##*/} ${ALL##*/}"
		$PATCH "$UIDIR/".*_$lang.d/message/*.bmg \
			--patch "replace,$src" \
			--patch "replace,$ALL"
	    fi
	done
    fi
fi


#--- stage messages

for src in "$PDIR"/stage-*.txt
do
    [[ -s $src ]] || continue
    stage="${src##*/}"
    stage="${stage#*-}"
    stage="${stage%%.*}"

    ((verbose)) && printf "$FORMAT" patch "$stage" "${src##*/}"
    $PATCH "$UIDIR/".${stage}_*.d/message/*.bmg \
		--patch "replace,$src"
done


#--- create message files

((verbose)) && printf "$FORMAT" create szs "$UIDIR $fast"
wszst create --quiet --remove $fast "$UIDIR/".*.d --dest %P/%1-N.szs

